@@ -15,11 +15,13 @@ TT_MINUS = 'MINUS'
 TT_MUL = 'MUL'
 TT_DIV = 'DIV'
 TT_POW = 'POW'
+TT_MOD = 'MOD'
 TT_EQ = 'EQ'
 TT_PLUS_EQ = 'PLUS_EQ'
 TT_MIN_EQ = 'MIN_EQ'
 TT_MUL_EQ = "MUL_EQ"
 TT_DIV_EQ = "DIV_EQ"
+TT_MOD_EQ = "MOD_EQ"
 TT_LPAREN = 'LPAREN'
 TT_RPAREN = 'RPAREN'
 TT_LSQUARE = 'LSQUARE'
@@ -59,4 +61,6 @@ KEYWORDS = [
     'redi',
     'continua',
     'confringe',
+    'transiet',
+    'attolle'
 ]
