@@ -3,8 +3,10 @@
 #######################################
 
 import Celeratas.helper.tokens as toks
-from Celeratas.helper.errors import (AttrError, IndexingError, NamingError,
-                                     RecursingError, RTError, TypingError)
+from Celeratas.helper.errors import (AttrError, Error, IndexingError,
+                                     NamingError, RecursingError, RTError,
+                                     TypingError)
+from Celeratas.parser.nodes import StringNode
 
 from .RTResult import RTResult
 from .values import Bool, Dict, Function, List, Number, Numeral, String
@@ -90,13 +92,6 @@ class Interpreter:
             if res.error:
                 return res
 
-            if isinstance(key, List):
-                return res.failure(TypingError(
-                    key.pos_start, key.pos_end,
-                    "Key cannot be a list",
-                    context
-                ))
-
             value = res.register(self.visit(value, context))
             if res.error:
                 return res
@@ -237,22 +232,25 @@ class Interpreter:
                 element_to_change = var_to_change
 
                 for for_idx, idx_to_change in enumerate(idxes_to_change):
+                    idx_to_change = res.register(self.visit(idx_to_change, context))
+                    idx_to_change = idx_to_change.value
+
                     if isinstance(element_to_change, List):
-                        if idx_to_change.value > len(element_to_change.elements) - 1:
+                        if idx_to_change > len(element_to_change.elements) - 1:
                             return res.failure(IndexingError(
                                 node.pos_start, node.pos_end,
                                 'List index out of bounds',
                                 context
                             ))
                         if for_idx == len(idxes_to_change) - 1:
-                            element_to_change.elements[idx_to_change.value] = value
+                            element_to_change.elements[idx_to_change] = value
                         else:
-                            element_to_change = element_to_change.elements[idx_to_change.value]
+                            element_to_change = element_to_change.elements[idx_to_change]
                     elif isinstance(element_to_change, Dict) or (hasattr(element_to_change, "elements") and isinstance(element_to_change.elements, Dict)):
                         if for_idx == len(idxes_to_change) - 1:
-                            element_to_change.key_pairs[idx_to_change.value] = value
+                            element_to_change.key_pairs[idx_to_change] = value
                         else:
-                            element_to_change = element_to_change.key_pairs[idx_to_change.value]
+                            element_to_change = element_to_change.key_pairs[idx_to_change]
                     else:
                         return res.failure(IndexingError(
                             node.pos_start, node.pos_end,
@@ -281,6 +279,8 @@ class Interpreter:
                     value, error = old_value.multed_by(value)
                 elif assign_type.type == "DIV_EQ":
                     value, error = old_value.dived_by(value)
+                elif assign_type.type == "MOD_EQ":
+                    value, error = old_value.moded_by(value)
 
                 if error:
                     return res.failure(error)
@@ -306,6 +306,8 @@ class Interpreter:
             result, error = left.multed_by(right)
         elif node.op_tok.type == toks.TT_DIV:
             result, error = left.dived_by(right)
+        elif node.op_tok.type == toks.TT_MOD:
+            result, error = left.moded_by(right)
         elif node.op_tok.type == toks.TT_POW:
             result, error = left.powed_by(right)
         elif node.op_tok.type == toks.TT_EE:
@@ -460,7 +462,6 @@ class Interpreter:
 
             elements.append(value)
 
-        context.symbol_table.remove(node.var_name)
         return res.success(
             None if node.should_return_null else
             List(elements).set_context(context).set_pos(
@@ -505,6 +506,39 @@ class Interpreter:
                 node.pos_start, node.pos_end)
         )
 
+    def visit_RaiseNode(self, node, context):
+        res = RTResult()
+        # For pos_start and pos_end if error
+        error_to_raise = node.error_to_raise
+
+        error_name = node.error_to_raise.node_to_call.var_name_to_get
+        error_message = node.error_to_raise.arg_nodes
+        error_message_string = ""
+
+        for idx, value in enumerate(error_message.values()):
+            if not isinstance(value, StringNode):
+                return res.failure(RTError(
+                    error_to_raise.pos_start, error_to_raise.pos_end,
+                    'Argument to Exception must be a string',
+                    context
+                ))
+
+            # Two dot values bc need to unpack RTResult and Interpreter String Class
+            value = self.visit_StringNode(value, context).value.value
+            error_message_string += value
+
+            if idx < len(error_message) - 1:
+                error_message_string += ", "
+
+        if error_name not in ["Exception", "TypeError", "NameError", "IndexError", "ZeroDivisionError"]:
+            return res.failure(Error(
+                error_to_raise.pos_start, error_to_raise.pos_end,
+                'Exception type not supported',
+                context
+            ))
+
+        return res.failure(Error(error_to_raise.pos_start, error_to_raise.pos_end, error_name, error_message_string))
+
     def visit_FuncDefNode(self, node, context):
         res = RTResult()
 
@@ -529,7 +563,6 @@ class Interpreter:
         return res.success(func_value)
 
     def visit_CallNode(self, node, context):
-
         if self.recursion_depth > 100:
             return RTResult().failure(RecursingError(
                 node.pos_start, node.pos_end,
@@ -579,3 +612,6 @@ class Interpreter:
 
     def visit_BreakNode(self, node, context):
         return RTResult().success_break()
+
+    def visit_PassNode(self, node, context):
+        return RTResult().success(None)
