@@ -34,24 +34,24 @@ class Lexer:
         tokens = []
         self.start_of_statement = True
         # Start of statements is before character other than space/tab - After this is false, tabs and spaces are ignored by make_spaces() method
-        # Grammatical char is set to True is anything other than space/tab is read - This var updates start of statements at the end of while loop
+        # non_space_char is set to True if anything other than space/tab is read - This var updates start of statements at the end of while loop
         while self.current_char:
-            grammatical_char = True
+            non_space_char = True
             if self.current_char in ' \t':
                 new_toks, error = self.make_space(tokens)
                 if error:
                     return [], error
                 tokens = new_toks
-                grammatical_char = False
+                non_space_char = False
             elif self.current_char == '#':
                 self.skip_comment()
                 self.start_of_statement = True
-                grammatical_char = False
+                non_space_char = False
             elif self.current_char in ';\n':
                 tokens.append(Token(toks.TT_NEWLINE, pos_start=self.pos))
                 self.advance()
                 self.start_of_statement = True
-                grammatical_char = False
+                non_space_char = False
             elif self.current_char in constants.ROMAN_NUMERAL_CHARS:
                 token, error = self.make_numeral()
                 if error:
@@ -79,14 +79,17 @@ class Lexer:
                     return [], error
                 tokens.append(token)
             elif self.current_char == '+':
-                tokens.append(self.make_plus())
+                tokens.append(self.make_mult_toks(toks.TT_PLUS, toks.TT_PLUS_EQ, "="))
             elif self.current_char == '-':
-                tokens.append(self.make_minus())
+                tokens.append(self.make_mult_toks(toks.TT_MINUS, toks.TT_MIN_EQ, "="))
             elif self.current_char == '*':
-                tokens.append(self.make_mul())
+                tokens.append(self.make_mult_toks(toks.TT_MUL, toks.TT_MUL_EQ, "="))
                 self.advance()
             elif self.current_char == '/':
-                tokens.append(self.make_div())
+                tokens.append(self.make_mult_toks(toks.TT_DIV, toks.TT_DIV_EQ, "="))
+                self.advance()
+            elif self.current_char == "%":
+                tokens.append(self.make_mult_toks(toks.TT_MOD, toks.TT_MOD_EQ, "="))
                 self.advance()
             elif self.current_char == '^':
                 tokens.append(Token(toks.TT_POW, pos_start=self.pos))
@@ -117,9 +120,9 @@ class Lexer:
             elif self.current_char == '=':
                 tokens.append(self.make_equals())
             elif self.current_char == '<':
-                tokens.append(self.make_less_than())
+                tokens.append(self.make_mult_toks(toks.TT_LT, toks.TT_LTE, "="))
             elif self.current_char == '>':
-                tokens.append(self.make_greater_than())
+                tokens.append(self.make_mult_toks(toks.TT_GT, toks.TT_GTE, "="))
             elif self.current_char == ':':
                 tokens.append(Token(toks.TT_COLON, pos_start=self.pos))
                 self.advance()
@@ -133,7 +136,7 @@ class Lexer:
                 return [], IllegalCharError(pos_start, self.pos, "'" + char + "'")
 
             # When Grammatical Token is read, tabs/spaces from this point on will be ignored
-            if self.start_of_statement and grammatical_char:
+            if self.start_of_statement and non_space_char:
                 self.start_of_statement = False
 
         tokens.append(Token(toks.TT_EOF, pos_start=self.pos))
@@ -183,7 +186,7 @@ class Lexer:
             numeral_str += self.current_char
             self.advance()
         # Check if identifier
-        if self.current_char and self.current_char != " ":
+        if self.current_char and self.current_char in constants.LETTERS:
             # Return the identifier
             identifier, error = self.make_identifier(numeral_str)
             # Need this extra code because the position of the pos start of the identifier will be behind the pos start of the numeral
@@ -337,24 +340,6 @@ class Lexer:
 
         return Token(tok_type, pos_start=pos_start, pos_end=self.pos)
 
-    def make_plus(self):
-        return(self.make_mult_toks(toks.TT_PLUS, toks.TT_PLUS_EQ, "="))
-
-    def make_minus(self):
-        return self.make_mult_toks(toks.TT_MINUS, toks.TT_MIN_EQ, "=")
-
-    def make_mul(self):
-        return(self.make_mult_toks(toks.TT_MUL, toks.TT_MUL_EQ, "="))
-
-    def make_div(self):
-        return(self.make_mult_toks(toks.TT_DIV, toks.TT_DIV_EQ, "="))
-
-    def make_less_than(self):
-        return self.make_mult_toks(toks.TT_LT, toks.TT_LTE, "=")
-
-    def make_greater_than(self):
-        return self.make_mult_toks(toks.TT_GT, toks.TT_GTE, "=")
-
     def skip_comment(self):
         self.advance()
 
