@@ -1,3 +1,4 @@
+
 class NumberNode:
     def __init__(self, value, pos_start, pos_end):
         self.value = value
@@ -147,6 +148,14 @@ class WhileNode:
         self.pos_end = pos_end
 
 
+class RaiseNode:
+    def __init__(self, error_to_raise, pos_start, pos_end):
+        self.error_to_raise = error_to_raise
+
+        self.pos_start = pos_start
+        self.pos_end = pos_end
+
+
 class FuncDefNode:
     def __init__(self, func_name, args, body_node, should_auto_return, pos_start, pos_end):
         self.func_name = func_name
@@ -185,3 +194,9 @@ class BreakNode:
     def __init__(self, pos_start, pos_end):
         self.pos_start = pos_start
         self.pos_end = pos_end
+
+
+class PassNode:
+    def __init__(self, pos_start, pos_end):
+        self.pos_start = pos_start
+        self.pos_end = pos_end
