@@ -2,14 +2,20 @@
 # IMPORTS
 #######################################
 
+# Readline will make the up and down arrows cycle through history
+import readline
 import sys
 from datetime import datetime
 
+import inquirer
+
 import Celeratas.interpreter.constants as constants
+from Celeratas.helper.errors import InteractivePrompt
 
 from .interpreter.Context import Context
 from .interpreter.Interpreter import Interpreter
 from .interpreter.SymbolTable import SymbolTable
+from .interpreter.values import List
 from .lexer.Lexer import Lexer
 from .parser.Parser import Parser
 
@@ -31,6 +37,7 @@ global_symbol_table.set("adde", constants.BuiltInFunction.append)
 global_symbol_table.set("remove", constants.BuiltInFunction.pop)
 global_symbol_table.set("extende", constants.BuiltInFunction.extend)
 global_symbol_table.set("longitudo", constants.BuiltInFunction.len)
+global_symbol_table.set("finde", constants.BuiltInFunction.split)
 global_symbol_table.set("curre", constants.BuiltInFunction.run)
 #######################################
 # RUN SCRIPT FUNCTION
@@ -85,28 +92,50 @@ class Shell:
 
     def help_menu(self):
         while True:
-            print(
-                "\nWhat do you need help with?\n\n1: Keywords\n2: Syntax\n3: Built-In Functions\n4: Leave\n")
-            while True:
-                help_num = input()
-                if help_num in ["1", "2", "3", "4"]:
-                    break
-                else:
-                    print("\nPlease only type 1, 2, 3, or 4\n")
+            num_prompt = [
+                inquirer.List('num',
+                              message="What do you need help with?",
+                              choices=['Keywords', 'Syntax', 'Built-In Functions', 'Leave'],
+                              carousel=True,
+                              ),
+            ]
+            # try block to quit help menu on keyboard interupt but not overall program - it catches type error bc of inquirer jank
+            try:
+                num = inquirer.prompt(num_prompt)["num"]
+            except TypeError:
+                return None
 
-            if help_num == "1":
+            if num == "Keywords":
                 print('\nVerus -> True\nFalsus -> False\net -> and\naut -> or\nnon -> not\nsi -> if')
-                print('alioquinsi -> elif\nalioquin -> else\ntempta -> try\npraeter -> except\tam -> as\nnpro -> for')
-                print('ad -> to\ngradus -> step\ndum -> while\nopus -> task\nfinis -> end\nredi -> return')
-                print('continua -> continue\nconfringe -> break')
-            elif help_num == "2":
-                with open("src/helper/grammar.txt", "r") as grammar:
-                    print(f"\n{grammar.read()}")
-            elif help_num == "3":
-                print('\nscribe(value_to_print) -> print\ninitus() -> input\npurgo() -> clear\nest_numerus(value_to_check) -> is_number')
+                print('alioquinsi -> elif\nalioquin -> else\ntempta -> try\npraeter -> except\ntam -> as\npro -> for')
+                print('ad -> to\ngradus -> step\ndum -> while\nopus -> task\nredi -> return')
+                print('continua -> continue\nconfringe -> break\ntransiet -> pass\nattolle -> raise\n')
+            elif num == "Syntax":
+                with open("Celeratas/helper/grammar.txt", "r") as grammar:
+                    print(f"{grammar.read()}\n")
+            elif num == "Built-In Functions":
+                print('\nscribe(value_to_print) -> print\ninitus(message_to_print) -> input\npurgo() -> clear\nest_numerus(value_to_check) -> is_number')
                 print('est_filum(value_to_check) -> is_string\nest_album(value_to_check) -> is_list\nest_opus(value_to_check) -> is_function')
                 print('adde(value_to_add) -> append\nremove(value_to_remove) -> pop\nextende(list_to_extend) -> extend\nlongitudo(value_to_check) -> length')
-                print('curre(file_to_run) -> run')
+                print('finde(string_to_split, seperator) -> split\ncurre(file_to_run) -> run\n')
+            else:
+                break
+
+            continue_prompt = [
+                inquirer.List('continue',
+                              message="Do you need anything else?",
+                              choices=['I need more help!', 'I\'m all good!'],
+                              carousel=True,
+                              ),
+            ]
+
+            try:
+                continue_ = inquirer.prompt(continue_prompt)["continue"]
+            except TypeError:
+                return None
+
+            if continue_ == "I need more help!":
+                continue
             else:
                 break
 
@@ -116,27 +145,30 @@ class Shell:
 
     def get_result(self, fn, script, interactive):
         result, error = run_script(fn, script)
+        result = [x for x in result.elements if x is not None] if result else None
 
-        if hasattr(error, "interactive") and error.interactive:
-            return True
-        elif error:
-            print(error.as_string())
-        elif result:
-            result.elements = [x for x in result.elements if repr(x) != "None"]
-            if len(result.elements) == 0:
-                pass
-            elif len(result.elements) == 1:
-                print(repr(result.elements[0]))
+        if error:
+            if isinstance(error, InteractivePrompt):
+                return error.symbol
+            elif error:
+                print(error.as_string())
+
+        elif result and interactive:
+            if len(result) == 1:
+                print(repr(result[0]))
             else:
                 print(repr(result))
-        return False
+
+        return None
 
     #######################################
     # RUN FILE FROM CLI ARGS OR TAKE INPUT
     #######################################
 
     def start(self):
+        global_symbol_table.set("__args__", List(sys.argv[2:]) if len(sys.argv) > 2 else List([]))
         try:
+            # Read file from CLI args
             if len(sys.argv) > 1:
                 try:
                     fn = sys.argv[1]
@@ -148,35 +180,39 @@ class Shell:
                 except UnicodeDecodeError:
                     print(f"Can't open file {fn}: Invalid file format")
             else:
+                # Interactive Mode
                 time = self.get_time()
                 print(
-                    f"Celeritas versio unum (defalta, {time})\nScribe 'auxilium' auxilio")
+                    f"Celeratas versio unum.tres (defalta, {time})\nScribe 'auxilium' auxilio")
                 more_statements = False
                 statements_txt = ""
+                prompt_symbol = ""
 
                 while True:
-                    text = input("... " if more_statements else ">>> ")
+                    text = input(f"{prompt_symbol * 3} " if more_statements else ">>> ")
 
                     if text == "auxilium" and not more_statements:
                         self.help_menu()
                         continue
 
+                    # Blank input should be ignored while not being prompted '...' otherwise it should end the statement
                     if text.strip() == "":
                         if more_statements:
                             self.get_result(
-                                "<stdin>", statements_txt, interactive=False)
+                                "<stdin>", statements_txt, interactive=True)
                             more_statements = False
                             continue
                         else:
                             continue
 
                     if more_statements:
+                        # Add another line and tab to the new line and add it to the statement
                         statements_txt += f"\n{text.rjust(len(text) + 4)}"
                     else:
-                        error = self.get_result(
-                            "<stdin>", text, interactive=False)
+                        prompt_symbol = self.get_result(
+                            "<stdin>", text, interactive=True)
                         # Error is true when program throws an interactive error
-                        if error:
+                        if prompt_symbol:
                             more_statements = True
                             statements_txt = text
 
